@@ -20,6 +20,7 @@ import random
 import six
 
 from tenacity import _utils
+from tenacity import compat as _compat
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -27,7 +28,7 @@ class wait_base(object):
     """Abstract base class for wait strategies."""
 
     @abc.abstractmethod
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
+    def __call__(self, retry_state):
         pass
 
     def __add__(self, other):
@@ -46,7 +47,8 @@ class wait_fixed(wait_base):
     def __init__(self, wait):
         self.wait_fixed = wait
 
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
         return self.wait_fixed
 
 
@@ -60,26 +62,27 @@ class wait_none(wait_fixed):
 class wait_random(wait_base):
     """Wait strategy that waits a random amount of time between min/max."""
 
-    def __init__(self, min=0, max=1):
+    def __init__(self, min=0, max=1):  # noqa
         self.wait_random_min = min
         self.wait_random_max = max
 
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
-        return (self.wait_random_min
-                + (random.random()
-                   * (self.wait_random_max - self.wait_random_min)))
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
+        return (self.wait_random_min +
+                (random.random() *
+                 (self.wait_random_max - self.wait_random_min)))
 
 
 class wait_combine(wait_base):
     """Combine several waiting strategies."""
 
     def __init__(self, *strategies):
-        self.wait_funcs = strategies
+        self.wait_funcs = tuple(_compat.wait_func_accept_retry_state(strategy)
+                                for strategy in strategies)
 
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
-        return sum(map(
-            lambda x: x(previous_attempt_number, delay_since_first_attempt),
-            self.wait_funcs))
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
+        return sum(x(retry_state=retry_state) for x in self.wait_funcs)
 
 
 class wait_chain(wait_base):
@@ -99,13 +102,15 @@ class wait_chain(wait_base):
     """
 
     def __init__(self, *strategies):
-        self.strategies = list(strategies)
+        self.strategies = [_compat.wait_func_accept_retry_state(strategy)
+                           for strategy in strategies]
 
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
-        wait_func = self.strategies[0]
-        if len(self.strategies) > 1:
-            self.strategies.pop(0)
-        return wait_func(previous_attempt_number, delay_since_first_attempt)
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
+        wait_func_no = min(max(retry_state.attempt_number, 1),
+                           len(self.strategies))
+        wait_func = self.strategies[wait_func_no - 1]
+        return wait_func(retry_state=retry_state)
 
 
 class wait_incrementing(wait_base):
@@ -115,14 +120,15 @@ class wait_incrementing(wait_base):
     (and restricting the upper limit to some maximum value).
     """
 
-    def __init__(self, start=0, increment=100, max=_utils.MAX_WAIT):
+    def __init__(self, start=0, increment=100, max=_utils.MAX_WAIT):  # noqa
         self.start = start
         self.increment = increment
         self.max = max
 
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
         result = self.start + (
-            self.increment * (previous_attempt_number - 1)
+            self.increment * (retry_state.attempt_number - 1)
         )
         return max(0, min(result, self.max))
 
@@ -131,7 +137,7 @@ class wait_exponential(wait_base):
     """Wait strategy that applies exponential backoff.
 
     It allows for a customized multiplier and an ability to restrict the
-    upper limit to some maximum value.
+    upper and lower limits to some maximum and minimum value.
 
     The intervals are fixed (i.e. there is no jitter), so this strategy is
     suitable for balancing retries against latency when a required resource is
@@ -140,30 +146,32 @@ class wait_exponential(wait_base):
     wait_random_exponential for the latter case.
     """
 
-    def __init__(self, multiplier=1, max=_utils.MAX_WAIT, exp_base=2):
+    def __init__(self, multiplier=1, max=_utils.MAX_WAIT, exp_base=2, min=0):  # noqa
         self.multiplier = multiplier
+        self.min = min
         self.max = max
         self.exp_base = exp_base
 
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
         try:
-            exp = self.exp_base ** previous_attempt_number
+            exp = self.exp_base ** retry_state.attempt_number
             result = self.multiplier * exp
         except OverflowError:
             return self.max
-        return max(0, min(result, self.max))
+        return max(max(0, self.min), min(result, self.max))
 
 
 class wait_random_exponential(wait_exponential):
     """Random wait with exponentially widening window.
 
     An exponential backoff strategy used to mediate contention between multiple
-    unco-ordinated processes for a shared resource in distributed systems. This
+    uncoordinated processes for a shared resource in distributed systems. This
     is the sense in which "exponential backoff" is meant in e.g. Ethernet
     networking, and corresponds to the "Full Jitter" algorithm described in
     this blog post:
 
-    https://www.awsarchitectureblog.com/2015/03/backoff.html
+    https://aws.amazon.com/blogs/architecture/exponential-backoff-and-jitter/
 
     Each retry occurs at a random time in a geometrically expanding interval.
     It allows for a custom multiplier and an ability to restrict the upper
@@ -177,8 +185,11 @@ class wait_random_exponential(wait_exponential):
     When waiting for an unavailable resource to become available again, as
     opposed to trying to resolve contention for a shared resource, the
     wait_exponential strategy (which uses a fixed interval) may be preferable.
+
     """
-    def __call__(self, previous_attempt_number, delay_since_first_attempt):
+
+    @_compat.wait_dunder_call_accept_old_params
+    def __call__(self, retry_state):
         high = super(wait_random_exponential, self).__call__(
-            previous_attempt_number, delay_since_first_attempt)
+            retry_state=retry_state)
         return random.uniform(0, high)
